@@ -14,7 +14,7 @@
 
 """Support for downloading media from Google APIs."""
 
-import urllib3.response
+import urllib3.response  # type: ignore
 
 from google._async_resumable_media import _download
 from google._async_resumable_media import _helpers
