@@ -27,6 +27,7 @@ import os
 import random
 import re
 import sys
+import urllib.parse
 
 from google import resumable_media
 from google.resumable_media import _helpers
@@ -82,7 +83,7 @@ class UploadBase(object):
         self.upload_url = upload_url
         if headers is None:
             headers = {}
-        self._headers = headers
+        self._headers = _helpers._base_headers(headers)
         self._finished = False
         self._retry_strategy = common.RetryStrategy()
 
@@ -462,10 +463,17 @@ class ResumableUpload(UploadBase):
 
         self._stream = stream
         self._content_type = content_type
-        headers = {
-            _CONTENT_TYPE_HEADER: "application/json; charset=UTF-8",
-            "x-upload-content-type": content_type,
-        }
+
+        # Signed URL requires content type set directly - not through x-upload-content-type
+        parse_result = urllib.parse.urlparse(self.upload_url)
+        parsed_query = urllib.parse.parse_qs(parse_result.query)
+        if "x-goog-signature" in parsed_query or "X-Goog-Signature" in parsed_query:
+            headers = {_CONTENT_TYPE_HEADER: content_type}
+        else:
+            headers = {
+                _CONTENT_TYPE_HEADER: "application/json; charset=UTF-8",
+                "x-upload-content-type": content_type,
+            }
         # Set the total bytes if possible.
         if total_bytes is not None:
             self._total_bytes = total_bytes
@@ -647,7 +655,7 @@ class ResumableUpload(UploadBase):
         """
         self._invalid = True
 
-    def _process_response(self, response, bytes_sent):
+    def _process_resumable_response(self, response, bytes_sent):
         """Process the response from an HTTP request.
 
         This is everything that must be done after a request that doesn't
@@ -670,7 +678,7 @@ class ResumableUpload(UploadBase):
         """
         status_code = _helpers.require_status_code(
             response,
-            (http.client.OK, resumable_media.PERMANENT_REDIRECT),
+            (http.client.OK, http.client.PERMANENT_REDIRECT),
             self._get_status_code,
             callback=self._make_invalid,
         )
@@ -814,7 +822,7 @@ class ResumableUpload(UploadBase):
         .. _sans-I/O: https://sans-io.readthedocs.io/
         """
         _helpers.require_status_code(
-            response, (resumable_media.PERMANENT_REDIRECT,), self._get_status_code
+            response, (http.client.PERMANENT_REDIRECT,), self._get_status_code
         )
         headers = self._get_headers(response)
         if _helpers.RANGE_HEADER in headers:
