@@ -17,12 +17,12 @@ import io
 import sys
 
 import mock
-import pytest
+import pytest  # type: ignore
 
-from google import resumable_media
 from google.resumable_media import _helpers
 from google.resumable_media import _upload
 from google.resumable_media import common
+from google.resumable_media._helpers import _base_headers
 
 
 URL_PREFIX = "https://www.googleapis.com/upload/storage/v1/b/{BUCKET}/o"
@@ -39,7 +39,7 @@ class TestUploadBase(object):
     def test_constructor_defaults(self):
         upload = _upload.UploadBase(SIMPLE_URL)
         assert upload.upload_url == SIMPLE_URL
-        assert upload._headers == {}
+        assert upload._headers == _base_headers({})
         assert not upload._finished
         _check_retry_strategy(upload)
 
@@ -47,7 +47,7 @@ class TestUploadBase(object):
         headers = {"spin": "doctors"}
         upload = _upload.UploadBase(SIMPLE_URL, headers=headers)
         assert upload.upload_url == SIMPLE_URL
-        assert upload._headers is headers
+        assert upload._headers == _base_headers(headers)
         assert not upload._finished
         _check_retry_strategy(upload)
 
@@ -140,7 +140,7 @@ class TestSimpleUpload(object):
         assert method == "POST"
         assert url == SIMPLE_URL
         assert payload == data
-        assert headers == {"content-type": content_type}
+        assert headers == _base_headers({"content-type": content_type})
 
     def test__prepare_request_with_headers(self):
         headers = {"x-goog-cheetos": "spicy"}
@@ -152,9 +152,8 @@ class TestSimpleUpload(object):
         assert method == "POST"
         assert url == SIMPLE_URL
         assert payload == data
-        assert new_headers is headers
         expected = {"content-type": content_type, "x-goog-cheetos": "spicy"}
-        assert headers == expected
+        assert new_headers == _base_headers(expected)
 
     def test_transmit(self):
         upload = _upload.SimpleUpload(SIMPLE_URL)
@@ -168,7 +167,7 @@ class TestMultipartUpload(object):
     def test_constructor_defaults(self):
         upload = _upload.MultipartUpload(MULTIPART_URL)
         assert upload.upload_url == MULTIPART_URL
-        assert upload._headers == {}
+        assert upload._headers == _base_headers({})
         assert upload._checksum_type is None
         assert not upload._finished
         _check_retry_strategy(upload)
@@ -177,7 +176,7 @@ class TestMultipartUpload(object):
         headers = {"spin": "doctors"}
         upload = _upload.MultipartUpload(MULTIPART_URL, headers=headers, checksum="md5")
         assert upload.upload_url == MULTIPART_URL
-        assert upload._headers is headers
+        assert upload._headers == _base_headers(headers)
         assert upload._checksum_type == "md5"
         assert not upload._finished
         _check_retry_strategy(upload)
@@ -256,18 +255,19 @@ class TestMultipartUpload(object):
 
     def test__prepare_request(self):
         headers, multipart_type = self._prepare_request_helper()
-        assert headers == {"content-type": multipart_type}
+        assert headers == _base_headers({"content-type": multipart_type})
 
     def test__prepare_request_with_headers(self):
         headers = {"best": "shirt", "worst": "hat"}
         new_headers, multipart_type = self._prepare_request_helper(headers=headers)
-        assert new_headers is headers
-        expected_headers = {
-            "best": "shirt",
-            "content-type": multipart_type,
-            "worst": "hat",
-        }
-        assert expected_headers == headers
+        expected_headers = _base_headers(
+            {
+                "best": "shirt",
+                "content-type": multipart_type,
+                "worst": "hat",
+            }
+        )
+        assert new_headers == expected_headers
 
     @pytest.mark.parametrize("checksum", ["md5", "crc32c"])
     def test__prepare_request_with_checksum(self, checksum):
@@ -278,9 +278,11 @@ class TestMultipartUpload(object):
         headers, multipart_type = self._prepare_request_helper(
             checksum=checksum, expected_checksum=checksums[checksum]
         )
-        assert headers == {
-            "content-type": multipart_type,
-        }
+        assert headers == _base_headers(
+            {
+                "content-type": multipart_type,
+            }
+        )
 
     @pytest.mark.parametrize("checksum", ["md5", "crc32c"])
     def test__prepare_request_with_checksum_overwrite(self, checksum):
@@ -293,9 +295,11 @@ class TestMultipartUpload(object):
             expected_checksum=checksums[checksum],
             test_overwrite=True,
         )
-        assert headers == {
-            "content-type": multipart_type,
-        }
+        assert headers == _base_headers(
+            {
+                "content-type": multipart_type,
+            }
+        )
 
     def test_transmit(self):
         upload = _upload.MultipartUpload(MULTIPART_URL)
@@ -310,7 +314,7 @@ class TestResumableUpload(object):
         chunk_size = ONE_MB
         upload = _upload.ResumableUpload(RESUMABLE_URL, chunk_size)
         assert upload.upload_url == RESUMABLE_URL
-        assert upload._headers == {}
+        assert upload._headers == _base_headers({})
         assert not upload._finished
         _check_retry_strategy(upload)
         assert upload._chunk_size == chunk_size
@@ -394,12 +398,14 @@ class TestResumableUpload(object):
         upload._total_bytes = 8192
         assert upload.total_bytes == 8192
 
-    def _prepare_initiate_request_helper(self, upload_headers=None, **method_kwargs):
+    def _prepare_initiate_request_helper(
+        self, upload_url=RESUMABLE_URL, upload_headers=None, **method_kwargs
+    ):
         data = b"some really big big data."
         stream = io.BytesIO(data)
         metadata = {"name": "big-data-file.txt"}
 
-        upload = _upload.ResumableUpload(RESUMABLE_URL, ONE_MB, headers=upload_headers)
+        upload = _upload.ResumableUpload(upload_url, ONE_MB, headers=upload_headers)
         orig_headers = upload._headers.copy()
         # Check ``upload``-s state before.
         assert upload._stream is None
@@ -434,7 +440,22 @@ class TestResumableUpload(object):
             "x-upload-content-length": "{:d}".format(len(data)),
             "x-upload-content-type": BASIC_CONTENT,
         }
-        assert headers == expected_headers
+        assert headers == _base_headers(expected_headers)
+
+    def test_prepare_initiate_request_with_signed_url(self):
+        signed_urls = [
+            "https://storage.googleapis.com/b/o?x-goog-signature=123abc",
+            "https://storage.googleapis.com/b/o?X-Goog-Signature=123abc",
+        ]
+        for signed_url in signed_urls:
+            data, headers = self._prepare_initiate_request_helper(
+                upload_url=signed_url,
+            )
+            expected_headers = {
+                "content-type": BASIC_CONTENT,
+                "x-upload-content-length": "{:d}".format(len(data)),
+            }
+            assert headers == _base_headers(expected_headers)
 
     def test__prepare_initiate_request_with_headers(self):
         headers = {"caviar": "beluga", "top": "quark"}
@@ -448,7 +469,7 @@ class TestResumableUpload(object):
             "x-upload-content-length": "{:d}".format(len(data)),
             "x-upload-content-type": BASIC_CONTENT,
         }
-        assert new_headers == expected_headers
+        assert new_headers == _base_headers(expected_headers)
 
     def test__prepare_initiate_request_known_size(self):
         total_bytes = 25
@@ -459,7 +480,7 @@ class TestResumableUpload(object):
             "x-upload-content-length": "{:d}".format(total_bytes),
             "x-upload-content-type": BASIC_CONTENT,
         }
-        assert headers == expected_headers
+        assert headers == _base_headers(expected_headers)
 
     def test__prepare_initiate_request_unknown_size(self):
         _, headers = self._prepare_initiate_request_helper(stream_final=False)
@@ -467,7 +488,7 @@ class TestResumableUpload(object):
             "content-type": "application/json; charset=UTF-8",
             "x-upload-content-type": BASIC_CONTENT,
         }
-        assert headers == expected_headers
+        assert headers == _base_headers(expected_headers)
 
     def test__prepare_initiate_request_already_initiated(self):
         upload = _upload.ResumableUpload(RESUMABLE_URL, ONE_MB)
@@ -706,7 +727,7 @@ class TestResumableUpload(object):
         upload._make_invalid()
         assert upload.invalid
 
-    def test__process_response_bad_status(self):
+    def test__process_resumable_response_bad_status(self):
         upload = _upload.ResumableUpload(RESUMABLE_URL, ONE_MB)
         _fix_up_virtual(upload)
 
@@ -714,18 +735,18 @@ class TestResumableUpload(object):
         assert not upload.invalid
         response = _make_response(status_code=http.client.NOT_FOUND)
         with pytest.raises(common.InvalidResponse) as exc_info:
-            upload._process_response(response, None)
+            upload._process_resumable_response(response, None)
 
         error = exc_info.value
         assert error.response is response
         assert len(error.args) == 5
         assert error.args[1] == response.status_code
         assert error.args[3] == http.client.OK
-        assert error.args[4] == resumable_media.PERMANENT_REDIRECT
+        assert error.args[4] == http.client.PERMANENT_REDIRECT
         # Make sure the upload is invalid after the failure.
         assert upload.invalid
 
-    def test__process_response_success(self):
+    def test__process_resumable_response_success(self):
         upload = _upload.ResumableUpload(RESUMABLE_URL, ONE_MB)
         _fix_up_virtual(upload)
 
@@ -744,21 +765,21 @@ class TestResumableUpload(object):
             status_code=http.client.OK,
             spec=["content", "status_code"],
         )
-        ret_val = upload._process_response(response, bytes_sent)
+        ret_val = upload._process_resumable_response(response, bytes_sent)
         assert ret_val is None
         # Check status after.
         assert upload._bytes_uploaded == total_bytes
         assert upload._finished
 
-    def test__process_response_partial_no_range(self):
+    def test__process_resumable_response_partial_no_range(self):
         upload = _upload.ResumableUpload(RESUMABLE_URL, ONE_MB)
         _fix_up_virtual(upload)
 
-        response = _make_response(status_code=resumable_media.PERMANENT_REDIRECT)
+        response = _make_response(status_code=http.client.PERMANENT_REDIRECT)
         # Make sure the upload is valid before the failure.
         assert not upload.invalid
         with pytest.raises(common.InvalidResponse) as exc_info:
-            upload._process_response(response, None)
+            upload._process_resumable_response(response, None)
         # Make sure the upload is invalid after the failure.
         assert upload.invalid
 
@@ -768,7 +789,7 @@ class TestResumableUpload(object):
         assert len(error.args) == 2
         assert error.args[1] == "range"
 
-    def test__process_response_partial_bad_range(self):
+    def test__process_resumable_response_partial_bad_range(self):
         upload = _upload.ResumableUpload(RESUMABLE_URL, ONE_MB)
         _fix_up_virtual(upload)
 
@@ -776,10 +797,10 @@ class TestResumableUpload(object):
         assert not upload.invalid
         headers = {"range": "nights 1-81"}
         response = _make_response(
-            status_code=resumable_media.PERMANENT_REDIRECT, headers=headers
+            status_code=http.client.PERMANENT_REDIRECT, headers=headers
         )
         with pytest.raises(common.InvalidResponse) as exc_info:
-            upload._process_response(response, 81)
+            upload._process_resumable_response(response, 81)
 
         # Check the error response.
         error = exc_info.value
@@ -789,7 +810,7 @@ class TestResumableUpload(object):
         # Make sure the upload is invalid after the failure.
         assert upload.invalid
 
-    def test__process_response_partial(self):
+    def test__process_resumable_response_partial(self):
         upload = _upload.ResumableUpload(RESUMABLE_URL, ONE_MB)
         _fix_up_virtual(upload)
 
@@ -797,9 +818,9 @@ class TestResumableUpload(object):
         assert upload._bytes_uploaded == 0
         headers = {"range": "bytes=0-171"}
         response = _make_response(
-            status_code=resumable_media.PERMANENT_REDIRECT, headers=headers
+            status_code=http.client.PERMANENT_REDIRECT, headers=headers
         )
-        ret_val = upload._process_response(response, 172)
+        ret_val = upload._process_resumable_response(response, 172)
         assert ret_val is None
         # Check status after.
         assert upload._bytes_uploaded == 172
@@ -944,7 +965,7 @@ class TestResumableUpload(object):
         assert payload is None
         assert headers == {"content-range": "bytes */*"}
         # Make sure headers are untouched.
-        assert upload._headers == {}
+        assert upload._headers == _base_headers({})
 
     def test__prepare_recover_request_with_headers(self):
         headers = {"lake": "ocean"}
@@ -959,7 +980,7 @@ class TestResumableUpload(object):
         # Make sure the ``_headers`` are not incorporated.
         assert "lake" not in new_headers
         # Make sure headers are untouched.
-        assert upload._headers == {"lake": "ocean"}
+        assert upload._headers == _base_headers({"lake": "ocean"})
 
     def test__process_recover_response_bad_status(self):
         upload = _upload.ResumableUpload(RESUMABLE_URL, ONE_MB)
@@ -975,7 +996,7 @@ class TestResumableUpload(object):
         assert error.response is response
         assert len(error.args) == 4
         assert error.args[1] == response.status_code
-        assert error.args[3] == resumable_media.PERMANENT_REDIRECT
+        assert error.args[3] == http.client.PERMANENT_REDIRECT
         # Make sure still invalid.
         assert upload.invalid
 
@@ -988,7 +1009,7 @@ class TestResumableUpload(object):
         upload._bytes_uploaded = mock.sentinel.not_zero
         assert upload.bytes_uploaded != 0
 
-        response = _make_response(status_code=resumable_media.PERMANENT_REDIRECT)
+        response = _make_response(status_code=http.client.PERMANENT_REDIRECT)
         ret_val = upload._process_recover_response(response)
         assert ret_val is None
         # Check the state of ``upload`` after.
@@ -1006,7 +1027,7 @@ class TestResumableUpload(object):
 
         headers = {"range": "bites=9-11"}
         response = _make_response(
-            status_code=resumable_media.PERMANENT_REDIRECT, headers=headers
+            status_code=http.client.PERMANENT_REDIRECT, headers=headers
         )
         with pytest.raises(common.InvalidResponse) as exc_info:
             upload._process_recover_response(response)
@@ -1032,7 +1053,7 @@ class TestResumableUpload(object):
         end = 11
         headers = {"range": "bytes=0-{:d}".format(end)}
         response = _make_response(
-            status_code=resumable_media.PERMANENT_REDIRECT, headers=headers
+            status_code=http.client.PERMANENT_REDIRECT, headers=headers
         )
         ret_val = upload._process_recover_response(response)
         assert ret_val is None
